package com.yizhi.comment.application.feign.student;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import com.yizhi.comment.application.vo.CommentParam;
import com.yizhi.comment.application.vo.NewApiCommentVo;
import com.yizhi.comment.application.vo.PostsCommentVo;
import io.swagger.annotations.ApiParam;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@FeignClient(name = "comment", contextId = "StudentCommentClient")
public interface StudentCommentClient {


    @PostMapping("/remote/student/comment/getPage")
    public Page<NewApiCommentVo> getPage(@RequestBody CommentParam param);

    /**
     * 论坛  我收到的回复分页
     *
     * @param param
     * @return
     */
    @PostMapping("/remote/student/comment/myReceivedPage")
    public Page<PostsCommentVo> myReceivedPage(@RequestBody CommentParam param);

    /**
     * 论坛  我收到的回复分页--查看对话
     *
     * @param id
     * @return
     */
    @GetMapping("/remote/student/comment/catCommunication")
    public List<PostsCommentVo> catCommunication(@RequestParam("id") Long id);

    /**
     * 论坛  我发送的回复分页
     *
     * @param param
     * @return
     */
    @PostMapping("/remote/student/comment/mySendPage")
    public Page<PostsCommentVo> mySendPage(@RequestBody CommentParam param);

    /**
     * 我的管理--回复管理--回复列表
     *
     * @param type
     * @param postsId
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/remote/student/comment/getPageByPostsId")
    public Page<PostsCommentVo> getPageByPostsId(@ApiParam("type必传 ") @RequestParam("type") Integer type,
                                                 @ApiParam("帖子id ") @RequestParam("postsId") Long postsId,
                                                 @ApiParam("pageNo") @RequestParam("pageNo") Integer pageNo,
                                                 @ApiParam("pageSize ") @RequestParam("pageSize") Integer pageSize);

    /**
     * 论坛 评论（回复）--删除（下架） 包含子级回复
     *
     * @param id
     * @param type
     * @return
     */
    @GetMapping("/remote/student/comment/down")
    public Integer down(@RequestParam("id") Long id, @RequestParam("type") Integer type, @RequestParam("isInform") Boolean isInform);

    /**
     * 论坛的评论保存是可以保存评论附件的
     * @param comment
     * @return
     */
    @PostMapping("/remote/student/comment/save")
    public Integer studentSave(@RequestBody NewCommentVo comment);


    /**
     *  论坛我的回复--获取我的回复（收到的回复未查看的数量）
     * @param taskType
     * @param taskIds
     * @return
     */
    @GetMapping("/remote/student/comment/myNum")
    public Integer myNum(@RequestParam("taskType") Integer taskType,
                         @RequestParam(value = "taskId",required = false) List<Long> taskIds);

    /**
     * 根据评论id  查询评论及其下的回复
     * @param taskType
     * @param taskId
     * @param commentId
     * @param orderType
     * @param pageNo
     * @param pageSize
     * @return
     */
    @GetMapping("/remote/student/comment/getCommentDetails")
    public NewApiCommentVo getCommentDetails(@ApiParam("taskType必传 ") @RequestParam("taskType") Integer taskType,
                                             @ApiParam("业务id ") @RequestParam("taskId") Long taskId,
                                             @ApiParam("评论id ") @RequestParam("commentId") Long commentId,
                                             @ApiParam("作者id ") @RequestParam(value = "author",required = false) Long author,
                                             @ApiParam("回复排序方式 ") @RequestParam("orderType") Integer orderType,
                                             @ApiParam("回复的pageNo") @RequestParam("pageNo") Integer pageNo,
                                             @ApiParam("回复的pageSize ") @RequestParam("pageSize") Integer pageSize);
}
