package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.NewComment;
import com.yizhi.application.domain.NewReply;
import com.yizhi.application.mapper.CommentMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.*;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.comment.application.vo.*;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import com.yizhi.comment.application.vo.domain.NewReplyVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.forum.application.feign.remote.RemotePostsClient;
import com.yizhi.forum.application.vo.forum.PostsParamsVo;
import com.yizhi.forum.application.vo.forum.PostsStudentVo;
import com.yizhi.point.application.feign.PointRedisFeignClients;
import com.yizhi.point.application.vo.PointParamVO;
import org.apache.ibatis.session.RowBounds;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 评论 服务实现类
 * </p>
 *
 * @author yizhi123
 * @since 2019-12-30
 */
@Service
public class CommentServiceImpl extends ServiceImpl<CommentMapper, NewComment> implements ICommentService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IReplyService replyService;
    @Autowired
    private ICommentAdmireService commentAdmireService;
    @Autowired
    private CommentMapper commentMapper;
    @Autowired
    private RemotePostsClient remotePostsClient;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private IInformRecordService informRecordService;
    @Autowired
    private ITaskAppendixService taskAppendixService;
    @Autowired
    private PointRedisFeignClients pointRedisFeignClients;
    /*@Autowired
    private AmqpTemplate amqpTemplate;*/

    private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private Logger logger = LoggerFactory.getLogger(CommentServiceImpl.class);



    @Override
    public Page<com.yizhi.comment.application.vo.CommentVo> getCommentPage(Long bizId, Integer bizType, int pageNo, int pageSize) {
        Page<com.yizhi.comment.application.vo.CommentVo> page = new Page<com.yizhi.comment.application.vo.CommentVo>(pageNo, pageSize);
        page.setRecords(
                this.baseMapper.getCommentPage(bizId, bizType, new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.getCommentPageCount(bizId, bizType);
        page.setTotal(count);
        return page;
    }

    @Override
    public List<com.yizhi.comment.application.vo.CommentVo> getCommentList(Long bizId, Integer bizType) {
        // TODO Auto-generated method stub
        List<CommentVo> list = this.baseMapper.getCommentList(bizId, bizType);
        return list;
    }

    @Override
    public Page<NewApiCommentVo> apiCommentPage(Long bizId, Integer bizType, Long accountId, int pageNo, int pageSize) {
        Page<NewApiCommentVo> page = new Page<NewApiCommentVo>(pageNo, pageSize);
        page.setRecords(this.baseMapper.apiCommentPage(bizId, bizType, accountId,
                new RowBounds(page.getOffset(), page.getLimit())));
        Integer count = this.baseMapper.apiCommentPageCount(bizId, bizType, accountId);
        if (count == null) {
            count = 0;
        }
        page.setTotal(count);
        return page;
    }

    @Override
    public Page<NewApiCommentVo> getPage(CommentParam param) {
        Page page = new Page<NewApiCommentVo>(param.getPageNo(), param.getPageSize());
        RequestContext context = ContextHolder.get();
        Map<Long, List<NewReplyVo>> replyMap = null;

        NewComment newCommentVo = new NewComment();
        newCommentVo.setBizType(param.getType());
        newCommentVo.setBizId(param.getBizId());
        newCommentVo.setState(Constant.UP);
        newCommentVo.setSiteId(context.getSiteId());
        EntityWrapper<NewComment> wrapper = new EntityWrapper(newCommentVo);
        //是否是最新回复
        wrapper.orderBy("create_time", Constant.ORDER_BY_OLD_REPLY.equals(param.getOrderType()));

        //只看楼主
        if (param.getOnlyAuthor()) {
            wrapper.andNew().eq("create_by_id", param.getAuthor());
            replyMap = replyService.getMapByAccountId(param.getType(), param.getBizId(), param.getAuthor());
            if (replyMap != null) {
                if (!CollectionUtils.isEmpty(replyMap.keySet())) {
                    wrapper.or().in("id", replyMap.keySet());
                }
            }
        }
        page = this.selectPage(page, wrapper);
        List<NewComment> records = page.getRecords();
        page.setRecords(buildVo(param, records, replyMap,context));

        return page;
    }
    /**
     * 判断是否为楼主
     *
     * @param replies
     * @param author
     * @return
     */
    public void isAuthor(List<NewReplyVo> replies, Long author) {

        if (!CollectionUtils.isEmpty(replies)) {
            replies.forEach(a -> {
                a.setIsAuthor(author.equals(a.getCreateById()));
            });
        }
    }

    /**
     * 评论填充vo数据
     *
     * @param param
     * @param records
     * @param replyMap
     * @return
     */
    public List<NewApiCommentVo> buildVo(CommentParam param, List<NewComment> records, Map<Long, List<NewReplyVo>> replyMap, RequestContext context) {
        if (!CollectionUtils.isEmpty(records)) {
            List<NewApiCommentVo> commentVos = new ArrayList<>(records.size());
            List<Long> commentIds = records.stream().map(a -> a.getId()).collect(Collectors.toList());
            //获取点赞相关数据
            Map<Long, AdmireVO> admireMap = commentAdmireService.getCountsByCommentId(Constant.TYPE_COMMENT, commentIds, false);
            //获取评论附件
            Map<Long, List<TaskAppendixVo>> appendixMap = taskAppendixService.getListMap(Constant.TYPE_COMMENT, commentIds);
            for (NewComment a : records) {
                NewApiCommentVo vo = new NewApiCommentVo();
                BeanUtils.copyProperties(a, vo);
                vo.setAccountId(a.getCreateById());
                //判断是否是楼主
                if (param.getAuthor() != null) {
                    vo.setIsAuthor(param.getAuthor().equals(a.getCreateById()));
                }
                //判断是否是当前用户创建的评论
                vo.setIsSelf(context.getAccountId().equals(a.getCreateById()));

                List<NewReplyVo> replies = null;
                if (admireMap != null) {
                    AdmireVO admireVO = admireMap.get(a.getId());
                    vo.setThumbsUp(0);
                    vo.setStatus(0);
                    if (admireVO != null) {
                        vo.setThumbsUp(admireVO.getAdmireNums());
                        vo.setStatus(admireVO.getHasAdmire()? 1 : 0);
                    }
                }
                if (appendixMap != null) {
                    vo.setTaskAppendixVos(appendixMap.get(a.getId()));
                }
                Page<NewReplyVo> replyPage = replyService.getPage(a.getId(),param);
                replies = replyPage.getRecords();
                vo.setReplies(replies);
                vo.setReplys(replyPage.getTotal());
                if (vo.getReplys() > param.getReplyPageSize()) {
                    vo.setRestReplys(vo.getReplys() - param.getReplyPageSize());
                } else {
                    vo.setRestReplys(0);
                }
                commentVos.add(vo);
            }
            return commentVos;
        }
        return new ArrayList<>();
    }

    @Override
    public Page<PostsCommentVo> myReceivedPage(CommentParam param) {
        RequestContext context = ContextHolder.get();
        Page<PostsCommentVo> page = new Page(param.getPageNo(), param.getPageSize());

        String keyWord = null;
        if (!StringUtils.isEmpty(param.getKeyWord())) {
            keyWord = param.getKeyWord();
        }

        PostsParamsVo vo = new PostsParamsVo();
        vo.setAuthor(context.getAccountId());
        vo.setIncludeDeleted(true);
        Map<Long, PostsStudentVo> map = remotePostsClient.getMap(vo);
        //获取作者已发帖ids
        List<Long> authorTaskIds = null;
        List<Long> keyWordPostsIds = null;
        if (map != null) {
            authorTaskIds = new ArrayList<>(map.size());
            authorTaskIds.addAll(map.keySet());
        }
        if (!StringUtils.isEmpty(param.getKeyWord())) {
            //获取模糊查询到帖子ids
            vo.setAuthor(null);
            vo.setTitle(param.getKeyWord());
            Map<Long, PostsStudentVo> map1 = remotePostsClient.getMap(vo);
            if (map1 != null) {
                keyWordPostsIds = new ArrayList<>(map1.size());
                keyWordPostsIds.addAll(map1.keySet());
            }
        }

        List<PostsCommentVo> records = commentMapper.myReceivedPage(param.getType(), keyWord, authorTaskIds, keyWordPostsIds, context.getAccountId(), new RowBounds(page.getOffset(), page.getLimit()));
        Integer total = commentMapper.myReceivedPageNum(param.getType(), keyWord, authorTaskIds, keyWordPostsIds, context.getAccountId());
        buildRecords(records);
        if (Constant.TASK_TYPE_POSTS.equals(param.getType())) {
            this.updateIsReadField(Constant.TASK_TYPE_POSTS, authorTaskIds);
        }
        page.setRecords(records);
        page.setTotal(total);
        return page;
    }

    /**
     * 组装数据
     *
     * @param records
     */
    public void buildRecords(List<PostsCommentVo> records) {
        if (!CollectionUtils.isEmpty(records)) {
            List<Long> admireCommentIds = new ArrayList<>();
            List<Long> commentIds = new ArrayList<>();
            List<Long> replyIds = new ArrayList<>();

            for (PostsCommentVo a : records) {
                //获取回复帖子的评论id,方便查询是否自己已点赞
                if (Constant.TYPE_COMMENT.equals(a.getType())){
                    admireCommentIds.add(a.getCommentId());
                }
                //获取回复回复的评论id（一级回复，查询该评论是否被删除）、父回复id(查询父回复是否被删除)
                if (Constant.TYPE_REPLY.equals(a.getType())) {
                    commentIds.add(a.getCommentId());
                    //若该数据是第N级回复（排除第二级）
                    if (!Constant.ZERO.equals(a.getReplyParentId())) {
                        replyIds.add(a.getReplyParentId());
                    }
                }
            }
            Map<Long, NewCommentVo> commentMap = null;
            Map<Long, NewReplyVo> replyMap = null;
            Map<Long, AdmireVO> admireMap = null;
            Map<Long, List<TaskAppendixVo>> appendixMap = null;
            if (!CollectionUtils.isEmpty(admireCommentIds)){
                //获取点赞数据
                admireMap = commentAdmireService.getCountsByCommentId(Constant.TYPE_COMMENT, admireCommentIds, false);
                //获取评论附件
                appendixMap = taskAppendixService.getListMap(Constant.TYPE_COMMENT, admireCommentIds);
            }
            if (!CollectionUtils.isEmpty(commentIds)){
                commentMap = this.getMap(commentIds);
            }
            if (!CollectionUtils.isEmpty(replyIds)){
                replyMap = replyService.getMap(replyIds);
            }
            for (PostsCommentVo a : records) {

                //判断是否自己已点赞
                if (Constant.TYPE_COMMENT.equals(a.getType())) {
                    if (admireMap != null) {
                       a.setIsAdmire(admireMap.containsKey(a.getCommentId()));
                    }
                    if (appendixMap != null) {
                        a.setTaskAppendixVos(appendixMap.get(a.getCommentId()));
                    }
                }
                if (Constant.TYPE_REPLY.equals(a.getType())) {
                    if (commentMap != null) {
                        NewCommentVo newCommentVo = commentMap.get(a.getCommentId());
                        //判断一级回复（评论）是否被删除
                        a.setCommentHasDeleted(newCommentVo == null ? false : newCommentVo.getState() != Constant.UP ? true : false);
                    }
                    //如果该数据是二级回复
                    if (Constant.ZERO.equals(a.getReplyParentId().intValue())) {
                        if (commentMap != null) {
                            NewCommentVo newCommentVo = commentMap.get(a.getCommentId());
                            a.setBeAnswerContent(newCommentVo == null ? "" : newCommentVo.getContent());
                            a.setBeAnswerContentHasDelete(newCommentVo == null ? false : newCommentVo.getState() != Constant.UP ? true : false);
                        }
                    } else {
                        //若该数据是第N级回复（排除第二级）
                        if (replyMap != null) {
                            NewReplyVo newReplyVo = replyMap.get(a.getReplyParentId());
                            a.setBeAnswerContent(newReplyVo == null ? "" : newReplyVo.getContent());
                            a.setBeAnswerContentHasDelete(newReplyVo == null ? false : newReplyVo.getState() != Constant.UP ? true : false);
                        }
                    }
                }
            }
        }
    }

    public static void main(String[] args) {
        Integer a =0;

    }

    @Override
    public Map<Long, NewCommentVo> getMap(List<Long> ids) {
        Map<Long, NewCommentVo> map = null;
        List<NewComment> newCommentVos = this.selectBatchIds(ids);
        if (!CollectionUtils.isEmpty(newCommentVos)) {
            map = new HashMap<>(newCommentVos.size());
            for (NewComment a : newCommentVos) {
                if (!map.containsKey(a.getId())) {
                    NewCommentVo nv=new NewCommentVo();
                    BeanUtils.copyProperties(a,nv);
                    map.put(a.getId(), nv);
                }
            }
        }
        return map;
    }


    @Override
    public Page<PostsCommentVo> mySendPage(CommentParam param) {
        RequestContext context = ContextHolder.get();
        Page<PostsCommentVo> page = new Page(param.getPageNo(), param.getPageSize());
        //获取包含关键字的帖子ids
        List<Long> taskIds = null;
        if (!StringUtils.isEmpty(param.getKeyWord())) {
            PostsParamsVo vo = new PostsParamsVo();
            vo.setIncludeDeleted(true);
            vo.setTitle(param.getKeyWord());
            Map<Long, PostsStudentVo> map = remotePostsClient.getMap(vo);
            if (map != null) {
                taskIds = new ArrayList<>(map.size());
                taskIds.addAll(map.keySet());
            }
        }
        List<PostsCommentVo> records = commentMapper.mySendPage(param.getType(), param.getKeyWord(), taskIds, context.getAccountId(), new RowBounds(page.getOffset(), page.getLimit()));
        Integer total = commentMapper.mySendPageNum(param.getType(), param.getKeyWord(), taskIds, context.getAccountId());
        buildRecords(records);
        page.setRecords(records);
        page.setTotal(total);
        return page;
    }

    @Override
    public Page<PostsCommentVo> getPageByPostsId(Integer type, Long postsId, Integer pageNo, Integer pageSize) {
        Page<PostsCommentVo> page = new Page(pageNo, pageSize);
        List<PostsCommentVo> records = commentMapper.getPageByPostsId(type, postsId, new RowBounds(page.getOffset(), page.getLimit()));
        if (!CollectionUtils.isEmpty(records)) {
            Set<Long> commentIds = records.stream().map(a -> a.getCommentId()).collect(Collectors.toSet());
            Map<Long, List<TaskAppendixVo>> map = taskAppendixService.getListMap(Constant.TYPE_COMMENT, new ArrayList<>(commentIds));
            records.forEach(a -> {
                if (map!=null){
                    a.setTaskAppendixVos(map.get(a.getId()));
                }
            });
        }
        page.setRecords(records);
        page.setTotal(commentMapper.getPageByPostsIdNum(type, postsId));
        //修改帖子主体信息
        this.updatePostsInfo(postsId,type, ContextHolder.get().getSiteId(), new Date(), false,true);
        return page;
    }

    @Override
      // @param operationType 1-下架   2-删除
    public Integer down(Integer operationType, Long id) {
        RequestContext context = ContextHolder.get();
        Long accountId = context.getAccountId();
        String name = context.getAccountName();
        Date date = new Date();

        NewComment comment = this.selectById(id);
        if (comment != null) {
            comment.setState(operationType == 1 ? Constant.DOWN : Constant.STATE_DELETED);
            comment.setUpdateById(accountId);
            comment.setUpdateByName(name);
            comment.setUpdateTime(date);
            if (this.updateById(comment)) {
                NewReply reply = new NewReply();
                reply.setState(operationType == 1 ? Constant.DOWN : Constant.STATE_DELETED);
                EntityWrapper wrapper = new EntityWrapper();
                wrapper.eq("comment_id", id);
                replyService.update(reply, wrapper);
            }
            return 1;
        }
        return 0;
    }

    @Override
    public Page<ManageCommentVo> getPageToManage(Integer type, Long postsId, Integer pageNo, Integer pageSize) {
        Page<ManageCommentVo> page = new Page(pageNo, pageSize);
        List<ManageCommentVo> records = commentMapper.getPageToManage(type, postsId, new RowBounds(page.getOffset(), page.getLimit()));
        page.setRecords(records);
        page.setTotal(commentMapper.getPageToManageNum(type, postsId));
        if (!CollectionUtils.isEmpty(records)) {
            List<Long> commentIds = new ArrayList<>();
            records.forEach(a -> {
                if (a.getType().equals(1)) {
                    commentIds.add(a.getId());
                }
            });
            //获取点赞相关数据
            Map<Long, AdmireVO> admireMap = null;
            if (!CollectionUtils.isEmpty(commentIds)) {
                admireMap = commentAdmireService.getCountsByCommentId(Constant.TYPE_COMMENT, commentIds, true);
            }

            Integer i = 0;
            for (ManageCommentVo a : records) {
                a.setStatusString(a.getStatus() == 1 ? "已上架" : "已下架");
                a.setCreateTimeString(format.format(a.getCreateTime()));
                if (a.getType().equals(1)) {
                    a.setNum(++i);
                    a.setBeAnswerName("--");
                    if (admireMap != null) {
                        AdmireVO admireVO = admireMap.get(a.getId());
                        if (admireVO != null) {
                            a.setCommentAdmires(admireVO.getAdmireNums());
                        }
                    }
                }
            }
            //如果评论回复是对帖子的
            if (Constant.TASK_TYPE_POSTS.equals(type)) {
                taskExecutor.asynExecute(new AbstractTaskHandler() {
                    @Override
                    public void handle() {
                        //更新帖子的新回复字段
                        com.yizhi.forum.application.vo.domain.PostsVo vo = new com.yizhi.forum.application.vo.domain.PostsVo();
                        vo.setId(postsId);
                        vo.setNewCommentNum(0);
                        remotePostsClient.updateInfo(vo);
                    }
                });
            }
        }
        return page;
    }

    /**
     *
     * @param bizId
     * @param bizType
     * @param siteId
     * @param date
     * @param save 是否为保存
     * @param isManage 是否进行了管理
     */
    @Override
    public void updatePostsInfo(Long bizId, Integer bizType, Long siteId, Date date, Boolean save,Boolean isManage) {
        //如果业务类型对帖子的
        if (Constant.TASK_TYPE_POSTS.equals(bizType)) {
            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {

                    //更新帖子新回复、最新评论时间、评论数、回复数字段、排序字段
                    com.yizhi.forum.application.vo.domain.PostsVo info = remotePostsClient.getInfo(bizId);
                    if (info != null) {
                        com.yizhi.forum.application.vo.domain.PostsVo vo = new com.yizhi.forum.application.vo.domain.PostsVo();
                        vo.setId(bizId);
                        if (isManage){
                            //如果是代管员查看列表，需要把该字段置零
                            vo.setNewCommentNum(0);
                           remotePostsClient.updateInfo(vo);
                            return;
                        }

                        NewCommentVo comment1 = new NewCommentVo();
                        comment1.setBizId(bizId);
                        comment1.setBizType(bizType);
                        comment1.setSiteId(siteId);
                        EntityWrapper wrapper1 = new EntityWrapper(comment1);
                        wrapper1.ne("state",Constant.STATE_DELETED);
                        Integer commentCount = commentMapper.selectCount(wrapper1);

                        NewReplyVo newReplyVo = new NewReplyVo();
                        newReplyVo.setBizId(bizId);
                        newReplyVo.setBizType(bizType);
                        EntityWrapper wrapper2 = new EntityWrapper(newReplyVo);
                        wrapper2.ne("state",Constant.STATE_DELETED);
                        int replyCount = replyService.selectCount(wrapper2);


                        vo.setCommentNum(commentCount);
                        vo.setReplyNum(replyCount);
                        if (save) {
                            vo.setCommentTime(date);
                            vo.setManageSortTime(date);
                            vo.setNewCommentNum(info.getNewCommentNum() == null ? 1 : info.getNewCommentNum() + 1);
                            if (info.getIsTop() != 1) {
                                vo.setSortTime(date);
                            }
                        }
                        remotePostsClient.updateInfo(vo);
                    }
                }
            });
        }
    }

    @Override
    public Map<Long, StatisticsForumAccountVo> getPersonalDataMap(Integer bizType) {

        Map<Long, StatisticsForumAccountVo> map = new HashMap<>();

        NewCommentVo comment = new NewCommentVo();
        comment.setBizType(bizType);
        comment.setState(Constant.UP);
        EntityWrapper wrapper1 = new EntityWrapper(comment);
        List<NewCommentVo> newCommentVos = commentMapper.selectList(wrapper1);
        if (!CollectionUtils.isEmpty(newCommentVos)) {
            for (NewCommentVo a : newCommentVos) {
                if (!map.containsKey(a.getCreateById())) {
                    StatisticsForumAccountVo vo = new StatisticsForumAccountVo();
                    vo.setCommentNum(1);
                    map.put(a.getCreateById(), vo);
                } else {
                    StatisticsForumAccountVo vo1 = map.get(a.getCreateById());
                    if (vo1 != null) {
                        vo1.setCommentNum(vo1.getCommentNum() + 1);
                        map.put(a.getCreateById(), vo1);
                    }
                }
            }
        }
        replyService.getPersonalDataMap(bizType, map);
        commentAdmireService.getPersonalDataMap(bizType, map);
        informRecordService.getPersonalDataMap(bizType, map);
        return map;
    }

    @Override
    public Integer save(NewCommentVo comment) {
        RequestContext context = ContextHolder.get();
        Date date = new Date();
        Long id = idGenerator.generate();
        comment.setId(id);
        comment.setCreateById(context.getAccountId());
        comment.setCreateByName(context.getAccountName());
        comment.setCreateTime(date);
        comment.setSiteId(context.getSiteId());
        comment.setOrgId(context.getOrgId());
        NewComment n=new NewComment();
        BeanUtils.copyProperties(comment,n);
        if (this.insert(n)) {
            taskAppendixService.save(Constant.TYPE_COMMENT, id, comment.getTaskAppendixVos());
            //修改帖子主体信息
            this.updatePostsInfo(comment.getBizId(), comment.getBizType(), context.getSiteId(), date, true,false);
            //发积分
            sendPoint(comment.getBizType(),id,comment.getBizId(),context,comment.getContent());
            return 1;
        }
        return 0;
    }

    /**
     * 判断评论是否发积分
     *
     * @param taskType  业务类型
     * @param commentId 评论id
     * @param postsId   帖子id
     * @param context
     */

    public void sendPoint(Integer taskType, Long commentId, Long postsId, RequestContext context,String activityName) {
        //只有对论坛的评论可以发积分
        if (!Constant.TASK_TYPE_POSTS.equals(taskType)) {
            return;
        }

        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                //先判断当前用户是否已评论啦
                NewComment comment = new NewComment();
                comment.setBizType(taskType);
                comment.setBizId(postsId);
                comment.setSiteId(context.getSiteId());
                comment.setCreateById(context.getAccountId());
                int count = commentMapper.selectCount(new EntityWrapper<>(comment));
                if (count > 1) {
                    //说明已评论则不发积分
                    logger.info("用户："+comment.getCreateById()+"已经在帖子："+comment.getBizId()+"评论啦！");
                    return;
                }
                //论坛回帖发送积分
                PointParamVO pointParamVO = new PointParamVO();
                pointParamVO.setEventName("pointForum");
                pointParamVO.setActivityName(activityName);
                pointParamVO.setActivityType("发布论坛回复发放积分");
                pointParamVO.setActivitySource("论坛");
                pointParamVO.setCreatePointTime(new Date());
                pointParamVO.setSourceId(commentId);
                pointParamVO.setAccountId(context.getAccountId());
                pointParamVO.setSiteId(context.getSiteId());
                pointParamVO.setCompanyId(context.getCompanyId());
                pointParamVO.setAccountName(context.getAccountName());
                pointParamVO.setOrgId(context.getOrgId());
                pointParamVO.setOperatingPoint(0);
                com.yizhi.forum.application.vo.domain.PostsVo info = remotePostsClient.getInfo(postsId);
                //PostsVo info = remotePostsClient.getInfo(postsId);
                if (info != null) {
                    pointParamVO.setOperatingPoint(info.getCommentPoint());
                }
                String sitePointId = pointRedisFeignClients.addPointRedis(pointParamVO);
                // 发送积分
                if (!StringUtils.isEmpty(sitePointId)) {
                   // amqpTemplate.convertAndSend("forum", sitePointId);
                }
            }
        });
    }

    @Override
    public Integer updateIsReadField(Integer taskType, List<Long> taskIds) {
        RequestContext context = ContextHolder.get();
        taskExecutor.asynExecute(new AbstractTaskHandler() {
            @Override
            public void handle() {
                if (!CollectionUtils.isEmpty(taskIds)) {
                    NewComment comment = new NewComment();
                    comment.setIsRead(1);
                    EntityWrapper wrapper = new EntityWrapper();
                    wrapper.eq("biz_type", taskType)
                            .eq("site_id", context.getSiteId())
                            .eq("is_read", Constant.ZERO)
                            .in("biz_id", taskIds);
                    commentMapper.update(comment, wrapper);
                }

                NewReply reply = new NewReply();
                reply.setIsRead(1);
                EntityWrapper wrapper1 = new EntityWrapper();
                wrapper1.eq("biz_type", taskType)
                        .eq("is_read", Constant.ZERO)
                        .eq("commenter_id", context.getAccountId());
                replyService.update(reply, wrapper1);
            }
        });
        return 1;
    }

    @Override
    public Integer myNum(Integer taskType, List<Long> taskIds) {
        Integer totalNum = 0;
        RequestContext context = ContextHolder.get();

        if (!CollectionUtils.isEmpty(taskIds)) {
            NewCommentVo comment = new NewCommentVo();
            comment.setBizType(taskType);
            comment.setIsRead(Constant.ZERO);
            comment.setSiteId(context.getSiteId());
            EntityWrapper wrapper = new EntityWrapper(comment);
            wrapper.in("biz_id", taskIds);
            Integer commentNum = commentMapper.selectCount(wrapper);
            totalNum  += commentNum;
        }

        NewReplyVo reply = new NewReplyVo();
        reply.setBizType(taskType);
        reply.setIsRead(Constant.ZERO);
        reply.setCommenterId(context.getAccountId());
        EntityWrapper wrapper1 = new EntityWrapper(reply);
        int replyNum = replyService.selectCount(wrapper1);
        totalNum += replyNum;
        return totalNum;
    }


    @Override
    public NewApiCommentVo getCommentDetails(Integer taskType, Long taskId, Long commentId, Long author, Integer orderType, Integer pageNo, Integer pageSize) {
        NewComment newCommentVo = this.selectById(commentId);
        NewApiCommentVo vo = new NewApiCommentVo();
        if (newCommentVo != null) {
            BeanUtils.copyProperties(newCommentVo, vo);
            vo.setAccountId(newCommentVo.getCreateById());
            vo.setIsSelf(ContextHolder.get().getAccountId().equals(newCommentVo.getCreateById()));
            List<Long> commentIds = Arrays.asList(commentId);

            //获取点赞相关数据
            Map<Long, AdmireVO> admireMap = commentAdmireService.getCountsByCommentId(Constant.TYPE_COMMENT, commentIds, false);
            //获取评论附件
            Map<Long, List<TaskAppendixVo>> appendixMap = taskAppendixService.getListMap(Constant.TYPE_COMMENT, commentIds);
            if (admireMap != null) {
                AdmireVO admireVO = admireMap.get(commentId);
                if (admireVO != null) {
                    vo.setThumbsUp(admireVO.getAdmireNums());
                    vo.setStatus(admireVO.getHasAdmire() == true ? 1 : 0);
                }
            }
            if (appendixMap != null) {
                vo.setTaskAppendixVos(appendixMap.get(commentId));
            }
            CommentParam param = new CommentParam();
            param.setReplyPageNo(pageNo);
            param.setReplyPageSize(pageSize);
            param.setOrderType(orderType);
            param.setAuthor(author);
            Page<NewReplyVo> replyPage = replyService.getPage(commentId, param);
//            vo.setReplies(replyPage.getRecords());
            vo.setReplyPage(replyPage);
            vo.setReplys(replyPage.getTotal());
        }
        return vo;
    }
}
