package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.InformRecord;
import com.yizhi.application.domain.NewComment;
import com.yizhi.application.domain.NewReply;
import com.yizhi.application.mapper.InformRecordMapper;
import com.yizhi.application.service.ICommentService;
import com.yizhi.application.service.IReplyService;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.comment.application.vo.domain.InformRecordVo;
import com.yizhi.comment.application.vo.domain.NewReplyVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import com.yizhi.forum.application.feign.remote.RemotePostsClient;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.service.ITaskAppendixService;
import com.yizhi.application.service.IInformRecordService;
import com.yizhi.core.application.task.AbstractTaskHandler;
import com.yizhi.core.application.task.TaskExecutor;
import com.yizhi.comment.application.vo.InformCommentVo;
import com.yizhi.comment.application.vo.StatisticsForumAccountVo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author taoge123
 * @since 2020-01-16
 */
@Service
public class InformRecordServiceImpl extends ServiceImpl<InformRecordMapper, InformRecord> implements IInformRecordService {

    private Logger logger = LoggerFactory.getLogger(InformRecordServiceImpl.class);
    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private ITaskAppendixService taskAppendixService;
    @Autowired
    private IReplyService replyService;
    @Autowired
    private ICommentService commentService;
    @Autowired
    private TaskExecutor taskExecutor;
    @Autowired
    private RemotePostsClient remotePostsClient;
    @Autowired
    private InformRecordMapper informRecordMapper;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(com.yizhi.comment.application.vo.InformRecordVo informRecordVo) {
        RequestContext context = ContextHolder.get();

        Long id = idGenerator.generate();
        InformRecord informRecord = new InformRecord();
        BeanUtils.copyProperties(informRecordVo, informRecord);
        informRecord.setId(id);
        informRecord.setCreateById(context.getAccountId());
        informRecord.setCreateByName(context.getAccountName());
        informRecord.setCreateTime(new Date());
        informRecord.setSiteId(context.getSiteId());
        informRecord.setCompanyId(context.getCompanyId());
        informRecord.setOrgId(context.getOrgId());

        if (this.insert(informRecord)) {
            taskAppendixService.save(Constant.TYPE_INFORM, id, informRecordVo.getTaskAppendixVos());
            updatePostsInfo(informRecordVo.getTaskType(), informRecordVo.getTaskId(), false, true);
            return 1;
        }
        return 0;
    }

    @Override
    public Page<com.yizhi.comment.application.vo.InformRecordVo> getPage(Integer taskType, Long taskId, Integer type, Integer terminalType, Integer pageNo, Integer pageSize) {
        Page page = new Page<>(pageNo, pageSize);
        RequestContext context = ContextHolder.get();
        InformRecordVo informRecordVo = new InformRecordVo();

        informRecordVo.setTaskType(taskType);
        informRecordVo.setTaskId(taskId);
        informRecordVo.setStatus(terminalType == 1 ? null : Constant.UP);
        informRecordVo.setDeleted(Constant.NOT_DELETED);
        EntityWrapper wrapper = new EntityWrapper(informRecordVo);
        if (type != null) {
            if (type == 0) {
                wrapper.eq("type", 0);
            } else {
                wrapper.ne("type", 0);
            }
        }
        page = this.selectPage(page, wrapper);
        List<InformRecord> records = page.getRecords();
        if (!CollectionUtils.isEmpty(records)) {
            List<com.yizhi.comment.application.vo.InformRecordVo> informRecordVoVos = new ArrayList<>(records.size());
            records.forEach(a -> {
                com.yizhi.comment.application.vo.InformRecordVo vo = new com.yizhi.comment.application.vo.InformRecordVo();
                BeanUtils.copyProperties(a, vo);
                vo.setTaskAppendixVos(taskAppendixService.getList(Constant.TYPE_INFORM, a.getId()));
                //构建回复vo
                vo.setInformCommentVo(buildCommentVo(taskType, a.getType(), a.getCommentId()));
                informRecordVoVos.add(vo);
            });
            page.setRecords(informRecordVoVos);
            //更新帖子的举报字段
            updatePostsInfo(taskType, taskId, true, false);
        }
        return page;
    }

    /**
     * 举报或者移除举报 更新帖子信息
     *
     * @param bizType  2-论坛
     * @param bizId
     * @param isManage 是否是管理员管理啦
     * @param isSave   是否是保存啦
     */
    public void updatePostsInfo(Integer bizType, Long bizId, Boolean isManage, Boolean isSave) {
        //如果业务类型对帖子的
        if (Constant.TASK_TYPE_POSTS.equals(bizType)) {
            taskExecutor.asynExecute(new AbstractTaskHandler() {
                @Override
                public void handle() {
                    com.yizhi.forum.application.vo.domain.PostsVo vo = new com.yizhi.forum.application.vo.domain.PostsVo();
                    vo.setId(bizId);

                    if (isSave) {
                        vo.setHasNewInform(1);
                        vo.setManageSortTime(new Date());
                    }
                    if (isManage) {
                        vo.setHasNewInform(0);
                        remotePostsClient.updateInfo(vo);
                        return;
                    }
                    //更新帖子的举报数字段
                    InformRecordVo informRecordVo1 = new InformRecordVo();
                    informRecordVo1.setTaskId(bizId);
                    informRecordVo1.setTaskType(bizType);
                    informRecordVo1.setDeleted(Constant.NOT_DELETED);
                    EntityWrapper wrapper = new EntityWrapper(informRecordVo1);
                    Integer count = informRecordMapper.selectCount(wrapper);
                    vo.setInformNum(count);
                    vo.setHasNewInform(count == 0 ? 0 : 1);
                    remotePostsClient.updateInfo(vo);
                }
            });
        }
    }

    /**
     * 举报类型为评论，需要带上评论信息
     *
     * @param taskType
     * @param type     1-评论  2-回复
     * @return commentReplyId 评论或回复的id
     */
    public InformCommentVo buildCommentVo(Integer taskType, Integer type, Long commentReplyId) {

        InformCommentVo vo = null;
        //只有为举报的回复评论类型才执行
        if (Constant.TASK_TYPE_POSTS.equals(taskType)) {
            vo = new InformCommentVo();
            vo.setType(type);
            if (Constant.TYPE_COMMENT.equals(type)) {
                NewComment newCommentVo = commentService.selectById(commentReplyId);
                if (newCommentVo != null) {
                    vo.setId(newCommentVo.getId());
                    vo.setAccountId(newCommentVo.getCreateById());
                    vo.setContent(newCommentVo.getContent());
                    vo.setCreateTime(newCommentVo.getCreateTime());
                    vo.setCommentStatus(newCommentVo.getState());
                } else {
                    logger.info("评论id无效");
                }
            } else {
                NewReply newReplyVo = replyService.selectById(commentReplyId);
                if (newReplyVo != null) {
                    vo.setId(newReplyVo.getId());
                    vo.setAccountId(newReplyVo.getCreateById());
                    vo.setContent(newReplyVo.getContent());
                    vo.setCreateTime(newReplyVo.getCreateTime());
                    vo.setCommentStatus(newReplyVo.getState());
                } else {
                    logger.info("回复id无效");
                }
            }
        }
        return vo;
    }

    @Override
    public Integer cancel(Long id) {
        InformRecord informRecordVo = this.selectById(id);
        if (informRecordVo != null) {
            RequestContext context = ContextHolder.get();
            InformRecord record = new InformRecord();
            record.setId(id);
            record.setDeleted(Constant.DELETED);
            record.setUpdateById(context.getAccountId());
            record.setUpdateByName(context.getAccountName());
            record.setUpdateTime(new Date());
            if (this.updateById(record)) {
                updatePostsInfo(informRecordVo.getTaskType(), informRecordVo.getTaskId(), false, false);
                return 1;
            }
            return 0;
        } else {
            return -1;
        }
    }

    @Override
    public Map<Long, StatisticsForumAccountVo> getPersonalDataMap(Integer bizType, Map<Long, StatisticsForumAccountVo> map) {
        InformRecordVo informRecordVo1 = new InformRecordVo();
        informRecordVo1.setTaskType(bizType);
        informRecordVo1.setDeleted(Constant.NOT_DELETED);
        EntityWrapper wrapper = new EntityWrapper(informRecordVo1);
        List<InformRecordVo> informRecordVos = this.selectList(wrapper);
        if (!CollectionUtils.isEmpty(informRecordVos)) {
            for (InformRecordVo a : informRecordVos) {
                if (!map.containsKey(a.getCreateById())) {
                    StatisticsForumAccountVo vo = new StatisticsForumAccountVo();
                    vo.setInformNum(1);
                    map.put(a.getCreateById(), vo);
                } else {
                    StatisticsForumAccountVo vo1 = map.get(a.getCreateById());
                    if (vo1 != null) {
                        Integer informNum = vo1.getInformNum();
                        informNum = informNum == null ? 0 : informNum;
                        vo1.setInformNum(informNum + 1);
                        map.put(a.getCreateById(), vo1);
                    }
                }
            }
        }
        return map;
    }

    @Override
    public Integer cancelByCommentId(Integer taskType, Long taskId, Integer type, Long commentId) {

        RequestContext context = ContextHolder.get();
        InformRecord record = new InformRecord();
        record.setStatus(Constant.DOWN);
        record.setUpdateById(context.getAccountId());
        record.setUpdateByName(context.getAccountName());
        record.setUpdateTime(new Date());
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("task_type", taskType)
                .eq("type", type)
                .eq("deleted", Constant.NOT_DELETED);
        if (taskId != null) {
            wrapper.eq("task_id", taskId);
        }
        if (commentId != null) {
            wrapper.eq("comment_id", commentId);
        }
        if (this.update(record, wrapper)) {
            updatePostsInfo(taskType, taskId, false, false);
            return 1;
        }
        return 0;
    }

    //    @Override
//    public Map<Long, Integer> getCountsMap(Integer type) {
//        Map<Long, Integer> informMap = null;
//        InformRecordVo informRecord1 = new InformRecordVo();
//        informRecord1.setTaskType(type);
//        informRecord1.setDeleted(Constant.NOT_DELETED);
//        EntityWrapper wrapper = new EntityWrapper(informRecord1);
//        List<InformRecordVo> informRecords = this.selectList(wrapper);
//        if (!CollectionUtils.isEmpty(informRecords)) {
//            for (InformRecordVo a : informRecords) {
//                if (informMap.containsKey(a.getCreateById())) {
//                    Integer num = informMap.get(a.getCreateById());
//                    informMap.put(a.getCreateById(), num + 1);
//                } else {
//                    informMap.put(a.getCreateById(), 1);
//                }
//            }
//        }
//        return informMap;
//    }
}
