package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.NewComment;
import com.yizhi.application.domain.NewReply;
import com.yizhi.application.mapper.CommentMapper;
import com.yizhi.application.mapper.ReplyMapper;
import com.yizhi.application.service.IReplyService;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.comment.application.vo.domain.NewCommentVo;
import com.yizhi.comment.application.vo.domain.NewReplyVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.service.ITaskAppendixService;
import com.yizhi.comment.application.vo.CommentParam;
import com.yizhi.comment.application.vo.CommentVo;
import com.yizhi.comment.application.vo.PostsCommentVo;
import com.yizhi.comment.application.vo.StatisticsForumAccountVo;
import org.apache.ibatis.session.RowBounds;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * <p>
 * 回复 服务实现类
 * </p>
 *
 * @author yizhi123
 * @since 2019-12-30
 */
@Service
public class ReplyServiceImpl extends ServiceImpl<ReplyMapper, NewReply> implements IReplyService {

	@Autowired
	private CommentMapper commentMapper;
	@Autowired
	private ITaskAppendixService taskAppendixService;

	@Override
	public Page<CommentVo> getCommentPage(Long commentId, int pageNo, int pageSize) {
		Page<CommentVo> page = new Page<CommentVo>(pageNo, pageSize);
		List<CommentVo> arrayList = new ArrayList<CommentVo>();
		List<NewReply> list = this.baseMapper.getCommentPage(commentId, new RowBounds(page.getOffset(), page.getLimit()));
		NewComment comment = commentMapper.selectById(commentId);
		if(!CollectionUtils.isEmpty(list)) {
			for(NewReply reply:list) {
				CommentVo vo = new CommentVo();
				vo.setAccountId(reply.getCreateById());
				vo.setContent(reply.getContent());
				vo.setId(reply.getId());
				vo.setCreateTime(reply.getCreateTime());
				vo.setState(reply.getState());
				vo.setReplyId(reply.getReplyParentId()==0?comment.getCreateById():reply.getCommenterId());
				arrayList.add(vo);
			}
		}

		page.setRecords(arrayList);
		Integer count = this.baseMapper.getCommentPageCount(commentId);
		page.setTotal(count);
		return page;
	}

	@Override
	public List<NewReply> list(Long bizId) {
		return this.baseMapper.list(bizId);
	}


	@Override
	public Map<Long, Integer> getCountsByCommentId(List<Long> commentIds) {
		Map<Long, Integer> replyMap = null;
		if (!CollectionUtils.isEmpty(commentIds)) {
			NewReplyVo newReplyVo = new NewReplyVo();
			EntityWrapper<NewReply> wrapper = new EntityWrapper(newReplyVo);
			wrapper.in("comment_id", commentIds);
			newReplyVo.setState(Constant.UP);
			List<NewReply> newReplies = this.selectList(wrapper);
			if (!CollectionUtils.isEmpty(newReplies)) {
				replyMap = new HashMap<>(newReplies.size());
				for (NewReply a : newReplies) {
					if (replyMap.containsKey(a.getCommentId())) {
						Integer commentAdmireNum = replyMap.get(a.getCommentId());
						replyMap.put(a.getCommentId(), commentAdmireNum + 1);
					} else {
						replyMap.put(a.getCommentId(), 1);
					}
				}
			}
		}
		return replyMap;
	}


	@Override
	public Page<NewReplyVo> getPage(Long commentId, CommentParam param) {

		Page<NewReply> page = new Page<NewReply>(param.getReplyPageNo(), param.getReplyPageSize());


		NewReply newReplyVo = new NewReply();
		newReplyVo.setCommentId(commentId);
		newReplyVo.setState(Constant.UP);
		if (param.getOnlyAuthor()){
		newReplyVo.setCreateById(param.getAuthor());
		}

		EntityWrapper<NewReply> wrapper = new EntityWrapper(newReplyVo);
		//是否是最新回复
		wrapper.orderBy("create_time", Constant.ORDER_BY_OLD_REPLY.equals(param.getOrderType()));
		Page<NewReply> replyPage = this.selectPage(page, wrapper);
		Page<NewReplyVo> replyVoPage=new Page<>();
		BeanUtils.copyProperties(replyPage,replyVoPage);
		replyVoPage.setRecords(isAuthor(replyPage.getRecords(),param.getAuthor()));
		replyVoPage.setCondition(replyPage.getCondition());
		return replyVoPage;
	}

	@Override
	public Map<Long, List<NewReplyVo>> getMapByAccountId(Integer taskType, Long taskId, Long author) {
		Map<Long, List<NewReplyVo>> replyMap = null;

		NewReply newReplyVo = new NewReply();
		newReplyVo.setBizType(taskType);
		newReplyVo.setState(Constant.UP);
		newReplyVo.setBizId(taskId);
		newReplyVo.setCreateById(author);
		EntityWrapper<NewReply> wrapper = new EntityWrapper(newReplyVo);

		List<NewReply> newReplies = this.selectList(wrapper);
		if (!CollectionUtils.isEmpty(newReplies)) {
			replyMap = new HashMap<>(newReplies.size());
			for (NewReply a : newReplies) {
				a.setIsAuthor(true);
				if (!replyMap.containsKey(a.getCommentId())) {
					NewReplyVo nrv=new NewReplyVo();
					BeanUtils.copyProperties(a,nrv);
					replyMap.put(a.getCommentId(), Arrays.asList(nrv));
				}
			}
		}
		return replyMap;
	}

	/**
	 * 判断是否为楼主，判断是否为当前用户创建的回复
	 *
	 * @param replies
	 * @param author
	 * @return
	 */
	public List<NewReplyVo> isAuthor(List<NewReply> replies, Long author) {

	    RequestContext context = ContextHolder.get();
		if (!CollectionUtils.isEmpty(replies)) {
			if (author != null) {
				replies.forEach(a -> {
					a.setIsAuthor(author.equals(a.getCreateById()));
					a.setIsSelf(context.getAccountId().equals(a.getCreateById()));
				});
			}
		}
		List<NewReplyVo> list=new ArrayList<>();
		for (NewReply nr:replies
			 ) {
			NewReplyVo nrv=new NewReplyVo();
			BeanUtils.copyProperties(nr,nrv);
			list.add(nrv);
		}
		return list;
	}

	@Override
	public Map<Long, NewReplyVo> getMap(List<Long> ids) {
		Map<Long, NewReplyVo> map = null;
		List<NewReply> newReplies = this.selectBatchIds(ids);
		if (!CollectionUtils.isEmpty(newReplies)) {
			map = new HashMap<>(newReplies.size());
			for (NewReply a : newReplies) {
				if (!map.containsKey(a.getId())) {
					NewReplyVo nrv=new NewReplyVo();
					BeanUtils.copyProperties(a,nrv);
					map.put(a.getId(), nrv);
				}
			}
		}
		return map;
	}

	@Override
	public List<PostsCommentVo> catCommunication(Long id) {
		NewReply newReplyVo = this.selectById(id);
		List<PostsCommentVo> postsCommentVos = new ArrayList<>();
		if (newReplyVo != null) {
			//构建vo
			buildVo(newReplyVo,postsCommentVos);
			Long replyParentId = newReplyVo.getReplyParentId();
			//该数据为二级回复
			if (Constant.ZERO.equals(replyParentId)) {
				buildCommentToVo(newReplyVo,postsCommentVos);
			}else {
				//否则递归获取直到最顶层
				getListByReplyParentId(newReplyVo, postsCommentVos);
			}
		}
		return postsCommentVos;
	}

	public void buildVo(NewReply newReplyVo, List<PostsCommentVo> postsCommentVos){
		if (newReplyVo != null) {
			PostsCommentVo vo = new PostsCommentVo();
			vo.setId(newReplyVo.getId());
			vo.setCommentId(newReplyVo.getCommentId());
			vo.setPostsId(newReplyVo.getBizId());
			vo.setType(Constant.TYPE_REPLY);
			vo.setIsDeleted(!newReplyVo.getState().equals(Constant.UP));
			vo.setContent(vo.getIsDeleted() ? Constant.COMMENT_HAS_DELETED : newReplyVo.getContent());
			vo.setCreateTime(newReplyVo.getCreateTime());
			vo.setAccountId(newReplyVo.getCreateById());
            vo.setIsSelf(ContextHolder.get().getAccountId().equals(newReplyVo.getCreateById()));
            vo.setBeAnswerId(newReplyVo.getCommenterId());
			postsCommentVos.add(vo);
		}
	}

	/**
	 * 查询出评论
	 * @param newReplyVo
	 * @param postsCommentVos
	 */
	public void buildCommentToVo(NewReply newReplyVo, List<PostsCommentVo> postsCommentVos){
		Long replyParentId = newReplyVo.getReplyParentId();
		//该数据为二级回复
		if (replyParentId != null && Constant.ZERO.toString().equals(replyParentId.toString()) ) {
			NewComment newCommentVo = commentMapper.selectById(newReplyVo.getCommentId());
			PostsCommentVo vo = new PostsCommentVo();
			vo.setId(newCommentVo.getId());
			vo.setPostsId(newCommentVo.getBizId());
			vo.setCommentId(newCommentVo.getId());
			vo.setType(Constant.TYPE_COMMENT);
			vo.setIsDeleted(!newCommentVo.getState().equals(Constant.UP));
			vo.setContent(vo.getIsDeleted() ? Constant.COMMENT_HAS_DELETED : newCommentVo.getContent());
			vo.setCreateTime(newCommentVo.getCreateTime());
			vo.setAccountId(newCommentVo.getCreateById());
			vo.setIsSelf(ContextHolder.get().getAccountId().equals(newCommentVo.getCreateById()));
			vo.setTaskAppendixVos(taskAppendixService.getList(Constant.TYPE_COMMENT, newCommentVo.getId()));
			postsCommentVos.add(vo);
		}
	}

	/**
	 * 递归获取父类回复
	 * @param newReplyVo
	 * @param postsCommentVos
	 */
	public void getListByReplyParentId(NewReply newReplyVo, List<PostsCommentVo> postsCommentVos) {
		Long replyParentId = newReplyVo.getReplyParentId();
		if (replyParentId != null && !Constant.ZERO.toString().equals(replyParentId.toString()) ) {
			NewReply newReplyVo1 = this.selectById(replyParentId);
			if (newReplyVo1 != null) {
				buildVo(newReplyVo1, postsCommentVos);
				getListByReplyParentId(newReplyVo1, postsCommentVos);
			}
		} else {
			buildCommentToVo(newReplyVo, postsCommentVos);
		}
	}

	@Override
	public Integer down(Integer operationType,Long id) {
		RequestContext context = ContextHolder.get();
		Long accountId = context.getAccountId();
		String name = context.getAccountName();
		Date date = new Date();

		NewReply reply = this.selectById(id);
		if (reply != null) {
			reply.setState(operationType == 1 ? Constant.DOWN : Constant.STATE_DELETED);
			reply.setUpdateById(accountId);
			reply.setUpdateByName(name);
			reply.setUpdateTime(date);
			this.updateById(reply);
			return 1;
		}
		return 0;
	}

	@Override
	public Map<Long, StatisticsForumAccountVo> getPersonalDataMap(Integer bizType, Map<Long, StatisticsForumAccountVo> map) {

		NewReplyVo newReplyVo = new NewReplyVo();
		newReplyVo.setBizType(bizType);
		newReplyVo.setState(Constant.UP);
		EntityWrapper wrapper2 = new EntityWrapper(newReplyVo);
		List<NewReplyVo> replies = this.selectList(wrapper2);
		if (!CollectionUtils.isEmpty(replies)){
			for (NewReplyVo a : replies) {
				if (!map.containsKey(a.getCreateById())) {
					StatisticsForumAccountVo vo = new StatisticsForumAccountVo();
					vo.setReplyNum(1);
					map.put(a.getCreateById(), vo);
				} else {
					StatisticsForumAccountVo vo1 = map.get(a.getCreateById());
					if (vo1 != null) {
						Integer replyNum = vo1.getReplyNum();
						replyNum = replyNum == null ? 0 : replyNum;
						vo1.setReplyNum(replyNum + 1);
						map.put(a.getCreateById(), vo1);
					}
				}
			}
		}
		return map;
	}

//	public static void main(String[] args) {
//		Map<Long, StatisticsForumAccountVo> map = new HashMap<>();
//		List<NewReplyVo> replies = new ArrayList<>()
//		if (!CollectionUtils.isEmpty(replies)){
//			for (NewReplyVo a : replies) {
//				if (!map.containsKey(a.getCreateById())) {
//					StatisticsForumAccountVo vo = new StatisticsForumAccountVo();
//					vo.setReplyNum(1);
//					map.put(a.getCreateById(), vo);
//				} else {
//					StatisticsForumAccountVo vo1 = map.get(a.getCreateById());
//					if (vo1 != null) {
//						vo1.setReplyNum(vo1.getReplyNum() + 1);
//						map.put(a.getCreateById(), vo1);
//					}
//				}
//			}
//		}
//	}
}
