package com.yizhi.application.task;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.application.service.ICommentService;
import com.yizhi.comment.application.constans.Constant;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.forum.application.vo.domain.PostsForbiddenAccountVo;
import com.yizhi.forum.application.feign.PostsForbiddenAccountClient;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import com.yizhi.forum.application.feign.PostsForbiddenAccountClient;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.comment.application.vo.ManageCommentVo;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class ForumCommentExportAsync extends AbstractDefaultTask<String, Map<String, Object>> {

    private static final Logger logger = LoggerFactory.getLogger(ForumCommentExportAsync.class);

    @Autowired
    private ICommentService commentService;


    @Autowired
    private AccountClient accountClient;

    @Autowired
    private PostsForbiddenAccountClient postsForbiddenAccountClient;

    private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    private SimpleDateFormat format1 = new SimpleDateFormat("yyyyMMddHHmmss");

    @Override
    protected String execute(Map<String, Object> arg0) {

        Long taskId = (Long) arg0.get("currentTaskId");
        String serialNo = (String) arg0.get("serialNo");
        String taskName = (String) arg0.get("currentTaskName");
        Long accountId = Long.valueOf(arg0.get("accountId").toString());
        Date submitTime = new Date((Long) arg0.get("submitTime"));
        Long siteId = Long.valueOf(arg0.get("siteId").toString());
        Long companyId = Long.valueOf(arg0.get("companyId").toString());
        Long bizId = (Long) arg0.get("bizId");
        Integer bizType = (Integer) arg0.get("bizType");
        String bizName = (String) arg0.get("bizName");
        /**
         * 走异步任务
         */
        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, submitTime, siteId, companyId);
        working(taskContext);

        //获取单个帖子下的回复数据
        Page<ManageCommentVo> page = commentService.getPageToManage(Constant.TASK_TYPE_POSTS, bizId, 1, Integer.MAX_VALUE);
        if (page == null) {
            return "数据为空！";
        }
        List<ManageCommentVo> records = page.getRecords();
        if (CollectionUtils.isEmpty(records)) {
            return "数据为空！";
        }

        //获取禁言名单
        List<PostsForbiddenAccountVo> postsForbiddenAccountVoList = postsForbiddenAccountClient.getListByPostsId(Arrays.asList(bizId));
        List<Long> forbiddenAccountIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(postsForbiddenAccountVoList)) {
            forbiddenAccountIds = postsForbiddenAccountVoList.stream().map(a -> a.getAccountId()).collect(Collectors.toList());
        }
        /*List<PostsForbiddenAccount> forbiddenAccounts = postsForbiddenAccountClient.getListByPostsId(Arrays.asList(bizId));
        List<Long> forbiddenAccountIds = new ArrayList<>();
        if (!CollectionUtils.isEmpty(forbiddenAccounts)) {
            forbiddenAccountIds = forbiddenAccounts.stream().map(a -> a.getAccountId()).collect(Collectors.toList());
        }*/

        //实时获取用户名
        Set<Long> accountIds = new HashSet<>(records.size());
        records.forEach(a -> {
            accountIds.add(a.getAccountId());
        });
        Map<Long, AccountVO> accountMap = this.queryAccountByIds(new ArrayList<>(accountIds));

        for (ManageCommentVo a : records) {
            //判断是否被禁言
            if (!CollectionUtils.isEmpty(forbiddenAccountIds)) {
                if (forbiddenAccountIds.contains(a.getAccountId())) {
                    a.setStatus(3);
                    a.setStatusString("已禁言");
                }
            }
            //填充用户信息
            if (accountMap != null) {
                AccountVO accountVO = accountMap.get(a.getAccountId());
                if (accountVO != null) {
                    a.setFullName(accountVO.getFullName());
                    a.setCommentator(accountVO.getName());
                }
                AccountVO accountVO1 = accountMap.get(a.getBeAnswerId());
                if (accountVO1 != null) {
                    a.setBeAnswerName(accountVO1.getName());
                }
            }
        }

        //下载路径
        String upLoadUrl = null;
        try {
            //excel生成过程: excel-->sheet-->row-->cell

            // 第一步，创建一个Excel文件
            XSSFWorkbook wb = new XSSFWorkbook();
            String title = bizName + "回复信息" + format1.format(submitTime);

            // 第二步，在webbook中添加一个sheet,对应Excel文件中的sheet
            XSSFSheet sheet = wb.createSheet(title);
            XSSFFont font = wb.createFont();
//			font.setFontName("宋体");
//			font.setFontHeightInPoints((short) 11);
            font.setBold(true);
            XSSFCellStyle style = wb.createCellStyle();
            style.setFont(font);
            //居中格式
            style.setAlignment(HorizontalAlignment.CENTER);
            style.setVerticalAlignment(VerticalAlignment.CENTER);

            // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制short
            XSSFRow row = sheet.createRow((int) 0);
            // 合并单元格
            CellRangeAddress cra = new CellRangeAddress(0, 0, 0, 7); // 起始行, 终止行, 起始列, 终止列
            sheet.addMergedRegion(cra);
            XSSFCell cell = row.createCell((short) 0);
            cell.setCellValue(title);
            cell.setCellStyle(style);

            XSSFRow titleRow = sheet.createRow((int) 1);
//        row2.setRowStyle(style);
//        row2.setHeight((short) (14 * 20));
            // 第四步，创建单元格
            titleRow.createCell((short) 0).setCellValue("回复人");
            titleRow.createCell((short) 1).setCellValue("姓名");
            titleRow.createCell((short) 2).setCellValue("回复对象");
            titleRow.createCell((short) 3).setCellValue("回复内容");
            titleRow.createCell((short) 4).setCellValue("回复时间");
            titleRow.createCell((short) 5).setCellValue("点赞数");
            titleRow.createCell((short) 6).setCellValue("回复状态");

            for (int i = 0; i < records.size(); i++) {
                row = sheet.createRow(i + 2);
//            row.setRowStyle(style);
//            row.setHeight((short) (13 * 20));
                // 第四步，创建单元格，并设置值
                ManageCommentVo vo = records.get(i);
                if (vo != null) {
                    row.createCell((short) 0).setCellValue(null == vo.getCommentator() ? "" : vo.getCommentator());
                    row.createCell((short) 1).setCellValue(null == vo.getFullName() ? "" : vo.getFullName());
                    row.createCell((short) 2).setCellValue(null == vo.getBeAnswerName() ? "" : vo.getBeAnswerName());
                    row.createCell((short) 3).setCellValue(null == vo.getContent() ? "" : vo.getContent());
                    row.createCell((short) 4).setCellValue(null == vo.getCreateTimeString() ? "" : vo.getCreateTimeString());
                    row.createCell((short) 5).setCellValue(null == vo.getCommentAdmires() ? "0个" : vo.getCommentAdmires() + "个");
                    row.createCell((short) 6).setCellValue(null == vo.getStatusString() ? "" : vo.getStatusString());
                }
            }

            String requestPath = "D:\\";
//            String requestPath = FileConstant.SAVE_PATH;
            File fileDir = new File(requestPath);
            if (!fileDir.exists()) {
                fileDir.mkdir();
            }

            // 第五步，写入实体数据 实际应用中这些数据从数据库得到，
            StringBuffer fileNameSb = new StringBuffer().append(title).append(".xlsx");
            String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
            FileOutputStream os = null;
            File file = null;
            try {
                os = new FileOutputStream(path);
                wb.write(os);
                //阿里云返回url
                upLoadUrl = OssUpload.upload(path, fileNameSb.toString());
                file = new File(path);
                success(taskContext, "成功", upLoadUrl);
            } catch (Exception e1) {
                e1.printStackTrace();
                fail(taskContext, "写入数据到Excel的过程中或者上传到阿里云中发生错误" + e1.getMessage());
                logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误" + e1.getMessage());
            } finally {
                if (os != null) {
                    os.close();
                }
                if (wb != null) {
                    wb.close();
                }
                if (file != null) {
                    file.delete();
                }
            }
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            fail(taskContext, "评论信息导出过程中发生错误，请查看日志" + e.getMessage());
            logger.error("评论信息导出过程中发生错误，请查看日志" + e.getMessage());
        }
        return upLoadUrl;
    }

    /**
     * 返回《accountId,account》的map
     *
     * @param accountIds
     * @return
     */
    public Map<Long, AccountVO> queryAccountByIds(List<Long> accountIds) {
        if (!CollectionUtils.isEmpty(accountIds)) {
            Map<Long, AccountVO> map = new HashMap<>(accountIds.size());
            List<AccountVO> accountVOS = accountClient.findByIds(accountIds);
            //组装map 以免双重循环
            if (!CollectionUtils.isEmpty(accountVOS)) {
                accountVOS.forEach(a -> {
                    if (!map.containsKey(a.getId())) {
                        map.put(a.getId(), a);
                    }
                });
            }
            return map;
        }
        return null;
    }
}
